
__author__ = 'Lukas Leufen'
__date__ = '2020-07-17'


from mlair.helpers import to_list
from mlair.data_handler.station_preparation import DataHandlerSingleStation
from mlair.data_handler import DefaultDataHandler
import os

from typing import Union, List

number = Union[float, int]
num_or_list = Union[number, List[number]]


class DataHandlerNeighbors(DefaultDataHandler):

    def __init__(self, id_class, data_path, neighbors=None, min_length=0,
                 extreme_values: num_or_list = None, extremes_on_right_tail_only: bool = False):
        self.neighbors = to_list(neighbors) if neighbors is not None else []
        super().__init__(id_class, data_path, min_length=min_length, extreme_values=extreme_values,
                         extremes_on_right_tail_only=extremes_on_right_tail_only)

    @classmethod
    def build(cls, station, **kwargs):
        sp_keys = {k: kwargs[k] for k in cls._requirements if k in kwargs}
        sp = DataHandlerSingleStation(station, **sp_keys)
        n_list = []
        for neighbor in kwargs.get("neighbors", []):
            n_list.append(DataHandlerSingleStation(neighbor, **sp_keys))
        else:
            kwargs["neighbors"] = n_list if len(n_list) > 0 else None
        dp_args = {k: kwargs[k] for k in cls.own_args("id_class") if k in kwargs}
        return cls(sp, **dp_args)

    def _create_collection(self):
        return [self.id_class] + self.neighbors

    def get_coordinates(self, include_neighbors=False):
        neighbors = list(map(lambda n: n.get_coordinates(), self.neighbors)) if include_neighbors is True else []
        return [super(DataHandlerNeighbors, self).get_coordinates()].append(neighbors)


if __name__ == "__main__":

    a = DataHandlerNeighbors
    requirements = a.requirements()

    kwargs = {"path": os.path.join(os.path.dirname(os.path.abspath(__file__)), "testdata"),
              "station_type": None,
              "network": 'UBA',
              "sampling": 'daily',
              "target_dim": 'variables',
              "target_var": 'o3',
              "time_dim": 'datetime',
              "window_history_size": 7,
              "window_lead_time": 3,
              "neighbors": ["DEBW034"],
              "data_path": os.path.join(os.path.dirname(os.path.abspath(__file__)), "testdata"),
              "statistics_per_var":  {'o3': 'dma8eu', 'temp': 'maximum'},
              "transformation": None,}
    a_inst = a.build("DEBW011", **kwargs)
    print(a_inst)
